
(* SPAN - A Security Protocol Animator
   Copyright (C) 2005-2006 Yann Glouche and Thomas Genet

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU Library General Public License version 2, as
   published by the Free Software Foundation.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   See the GNU Library General Public License version 2 for more details
   (enclosed in the file LGPL).


   Version 1.0. Last modification 30/11/06
*)
(**
interface.ml
@author Yann GLOUCHE
@version 1.0
@since 12/12/2005
*)

open Tk;;

(** Initialize Tk *)
let root = openTk ();;

(** Colors *)

let defcol = (`Color "#dfdfdf" : [> `Color of string ]);;
let selcol = (`Color "#ffffff" : [> `Color of string ]);;
let clicol = (`Color "#ffafaf" : [> `Color of string ]);;
let blue = (`Color "#9999ff" : [> `Color of string ]);;

(** Verification Tools *)


(* For using files with pathes containing spaces we need an escape character *)

let esc="\""


(* If avispa tools are already installed then we use path to those tools and AVISPA_PACKAGE variable 
   otherwise we use tools included in span package *)


let path_span = 
  (try 
    Sys.getenv "SPAN"
  with Not_found -> failwith "Environment Variable SPAN should be declared!" )


let path_avispa =
  (try
    Sys.getenv "AVISPA_PACKAGE"
  with
    Not_found -> path_span)


let hlpsl2if = (path_avispa^"/bin/translator/hlpsl2if" : string);;
let ofmc = (path_avispa^"/bin/backends/ofmc/ofmc" : string);;
let satmc = (path_avispa^"/bin/backends/satmc.bash" : string);;
let atse = (path_avispa^"/bin/backends/cl/cl-atse" : string);;
let ta4sp = (path_avispa^"/bin/backends/ta4sp.bash" : string);;
let hlpsl2scenario = (path_span^"/bin/protocol_simulation/hlpsl2scenario" : string);;

if not ((Sys.file_exists hlpsl2if) &
	(Sys.file_exists hlpsl2scenario))
then failwith "Check your environment variables ($AVISPA_PACKAGE and $SPAN): one of hlpsl2if or hlpsl2scenario is not reachable"


(** repertory for the tempory files generated by the tools *)
let temp_dir = ( (path_avispa^"/testsuite/results/") : string);;

(** font text *)

let get_font (tag : string) (choices : string list) : Tk.font =
  let all = Font.families () in
  let rec find ch =
    match ch with
    | [] -> None
    | f::tail ->
      ( try Some (List.find (( = ) f) all)
        with Not_found -> find tail ) in
  match find choices with
  | None -> Font.create ~name:tag ~size:10 ()
  | Some fam -> Font.create ~name:tag ~family:fam ~size:10 ();;

let configure (() : unit) : unit =
  ignore(get_font "guifont" ["verdana"; "lucida"; "bitstream vera sans"]);
  ignore(get_font "textfont" ["lettergothic"; "lucida typewriter"; "bitstream vera sans mono"; "courier"]);
  Option.add ~path:"*Menu.font" "guifont";
  Option.add ~path:"*Label.font" "guifont";
  Option.add ~path:"*Message.font" "guifont";
  Option.add ~path:"*Button.font" "guifont";
  Option.add ~path:"*Radiobutton.font" "guifont";
  Option.add ~path:"*Checkbutton.font" "guifont";
  Option.add ~path:"*Text.font" "textfont";
  Option.add ~path:"*Canvas.font" "textfont";
  Option.add ~path:"*Listbox.font" "textfont";
  Option.add ~path:"*Entry.font" "textfont";
  Option.add ~path:"*Text.borderWidth" "1";
  Option.add ~path:"*Canvas.borderWidth" "1";
  Option.add ~path:"*Button.borderWidth" "1";
  Option.add ~path:"*Scrollbar.borderWidth" "1";
  Option.add ~path:"*Entry.borderWidth" "1";
  Option.add ~path:"*Listbox.borderWidth" "1";
  Option.add ~path:"*Menu.borderWidth" "1";
  Option.add ~path:"*Menu.activeBorderWidth" "1";
  Option.add ~path:"*Menu.tearOff" "false";
  Option.add ~path:"*Listbox.selectBorderWidth" "0";
  Option.add ~path:"*Listbox.selectBackground" "#0000a0";
  Option.add ~path:"*Listbox.selectForeground" "#ffffff";
  Option.add ~path:"*Listbox.background" "#ffffff";;

let words = ([
  "anaerobic"; "beneficent"; "corundum"; "diesel";
  "erotic"; "fragment"; "garrulous"; "hexagon";
  "irate"; "jinx"; "kerosene"; "lethargic";
  "malamute"; "normality"; "oblate"; "parsimony";
  "quail"; "rigorous"; "scimitar"; "tuxedo";
  "ululation"; "verify"; "warbler"; "xenon";
  "yawl"; "zither"
] : string list);;

let nop (() : unit) : unit = ();;

(** Read a file and return a list of string where each string is a line of the file 
@return string list
*)
let readFile (fileName : string) : string =
   let channel = open_in fileName in
   let res = ref "" in
   (try
   begin
    while true do
     let line = input_line channel
     in res:=(!res) ^ "\n" ^ line ; 
    done;
   end;
   with
   End_of_file ->
   begin
   close_in channel;
   end);
   !res;;


 (** Read a file and return the number of lines of the file with a list of string where each string is a line of the file 
@return string list
*)
let readFile_and_give_line_number (fileName : string) : string * int =
   let channel = open_in fileName in
   let res = ref "" in
   let line_number = ref 0 in
   try
   begin
    while true do
     let line = input_line channel
     in res:=(!res) ^ "\n" ^ line ; 
     line_number := (!line_number)+1;
    done;
   end;
   failwith "problem"
   with
   End_of_file ->
   begin
   close_in channel;
   ((!res),(!line_number));
   end;;

(** Save a file 
@param txt is the text to save
@return unit
*)
let saveFile (fileName : string) (txt : string) : unit =
   let channel = open_out fileName in
   output_string channel txt;
   close_out channel;;

let get_output_if (fileName : string) : string =
 if(Filename.check_suffix fileName ".hlpsl") 
 then (temp_dir^(Filename.basename ((Filename.chop_suffix fileName ".hlpsl")^".if")))
 else (temp_dir^(Filename.basename (fileName^".if")));;


class hlpsl2ifOption optionFrame =
object
 val optionLabel : Widget.label Widget.widget = Label.create ~text:"Options" ~width:20 ~height:2 ~background:blue optionFrame
 val types : Widget.checkbutton Widget.widget = Checkbutton.create ~text:"Types section  " ~width:20 ~height:2 ~borderwidth:1 optionFrame
 val mutable typesString : string = ""
 val inits :  Widget.checkbutton Widget.widget = Checkbutton.create ~text:"Inits section    " ~width:20 ~height:2 ~borderwidth:1 optionFrame
 val mutable initString : string = ""
 val rules : Widget.checkbutton Widget.widget = Checkbutton.create ~text:"Rules section  " ~width:20 ~height:2 ~borderwidth:1 optionFrame
 val mutable rulesString : string = ""
 val goals : Widget.checkbutton Widget.widget = Checkbutton.create ~text:"Goals section  " ~width:20 ~height:2 ~borderwidth:1 optionFrame
 val mutable goalsString : string = ""
 (** Initialize the objects of this class *)
 method init : unit = (
     Checkbutton.configure types ~command:(fun () -> (if typesString="" then typesString <- " --types " else typesString <- ""));
     Checkbutton.configure inits ~command:(fun () -> (if initString="" then initString <- " --init " else initString <- ""));
     Checkbutton.configure rules ~command:(fun () -> (if rulesString="" then rulesString <- " --rules " else rulesString <- ""));
     Checkbutton.configure goals ~command:(fun () -> (if goalsString="" then goalsString <- " --goals " else goalsString <- ""));
     pack ~side:`Top [optionFrame];
     pack ~side:`Top [optionLabel];
     pack ~side:`Top [types];
     pack ~side:`Top [inits];
     pack ~side:`Top [rules];
     pack ~side:`Top [goals];)
 (** Destruct the objects of this class, and the graphical objects builds by the objects of this class *)
 method destroy : unit = ( destroy optionLabel; 
		    destroy types; 
		    destroy inits; 
		    destroy rules; 
		    destroy goals;)
 (** Execute HLPSL2IF with the parameters of the user *)
 method execution (fileName : string) (outText : Widget.text Widget.widget) : unit = (
   let out_name = get_output_if fileName in
   let error_if_file = (out_name^".error") in
   if(Sys.file_exists out_name) then ignore(Sys.remove out_name);
   if(Sys.file_exists error_if_file) then ignore(Sys.remove error_if_file);
   ignore(Sys.command(hlpsl2if^" --stdout "^esc^fileName^esc^typesString^initString^rulesString^goalsString^" >"^out_name^" 2>"^error_if_file));
   let txt = if(Sys.file_exists out_name) then (
   		let txt = readFile out_name in
		(if(Sys.file_exists error_if_file) 
		then let (out_error, num_of_lines) = readFile_and_give_line_number error_if_file in
			if(num_of_lines >2)
			then ("\n----- Output error of if translator :\n\n"^out_error^(if(txt<>"") then ("\n--------------- \n"^txt) else ""))
			else txt;
		else txt)
		)
	      else (if(Sys.file_exists error_if_file) 
		    then ("\n----- Output error of if translator :\n\n"^(readFile error_if_file))
		    else "")
   in
   Text.delete outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`End,[]) ;
   Text.insert ~index:(`End,[]) ~text:txt outText;
   if(Sys.file_exists out_name) then ignore(Sys.remove out_name);
   if(Sys.file_exists error_if_file) then ignore(Sys.remove error_if_file);
   )
end;;



class ofmcOption optionFrame =
object
 val optionLabel : Widget.label Widget.widget = Label.create ~text:"Options" ~width:20 ~height:2 ~background:blue optionFrame
 val session : Widget.checkbutton Widget.widget = Checkbutton.create ~text:"Session Compilation" ~width:20 ~height:2 ~borderwidth:1 optionFrame
 val mutable sessionString : string = ""
 val f : Widget.frame Widget.widget = Frame.create optionFrame
 val mutable hf1 : Widget.frame Widget.widget = Frame.create optionFrame
 val mutable labelDepth : Widget.label Widget.widget = Label.create ~text:"Defth : " ~width:20 ~height:2 optionFrame
 val mutable entryDepth : Widget.entry Widget.widget = Entry.create ~width:20 ~borderwidth:1 optionFrame
 val mutable hf2 : Widget.frame Widget.widget = Frame.create optionFrame
 val mutable labelPath : Widget.label Widget.widget = Label.create ~text:"Path : " ~width:20 ~height:2 optionFrame
 val mutable entryPath : Widget.entry Widget.widget = Entry.create ~width:20 ~borderwidth:1 optionFrame
  (** Initialize the objects of this class *)
 method init : unit = (
 	destroy hf1;
 	hf1 <- Frame.create f;
	 destroy labelDepth;
	 labelDepth <- Label.create ~text:"Defth : " ~width:20 ~height:2 hf1;
	 destroy entryDepth;
	 entryDepth <- Entry.create ~width:20 ~background:selcol hf1;
	 destroy hf2;
	 hf2 <- Frame.create f;
	 destroy labelPath;
	 labelPath <- Label.create ~text:"Path : " ~width:20 ~height:2 hf2;
	 destroy entryPath;
	entryPath <- Entry.create ~width:20 ~background:selcol hf2;
	Checkbutton.configure session ~command:(fun () -> (if sessionString="" then sessionString <- " -sessco " else sessionString <- ""));  
	pack ~side:`Top [optionFrame];
	pack ~side:`Top [optionLabel];
	pack ~side:`Top [session];
	pack ~side:`Top [f];
	pack ~side:`Top [hf1];
	pack ~side:`Left [labelDepth];
	pack ~side:`Left [entryDepth];
	pack ~side:`Top [hf2];
	pack ~side:`Left [labelPath];
     	pack ~side:`Left [entryPath];)
 (** Destruct the objects of this class, and the graphical objects builds by the objects of this class *)
 method destroy : unit = ( destroy optionLabel; destroy session; 
		    destroy f; destroy hf1; destroy labelDepth; 
		    destroy entryDepth; destroy hf2;
		    destroy labelPath; destroy entryPath;)
 (** Execute OFMC with the parameters of the user *)
 method execution (fileName : string) (outText : Widget.text Widget.widget) : unit = (
   let out_name_if = get_output_if fileName in
   let out_name_ofmc = (temp_dir^(Filename.basename fileName)^".ofmc") in
   let error_if_file = (out_name_if^".error") in
   let error_ofmc_file = (out_name_ofmc^".error") in
   if(Sys.file_exists out_name_if) then ignore(Sys.remove out_name_if);
   if(Sys.file_exists out_name_ofmc) then ignore(Sys.remove out_name_ofmc);
   if(Sys.file_exists error_if_file) then ignore(Sys.remove error_if_file);
   if(Sys.file_exists error_ofmc_file) then ignore(Sys.remove error_ofmc_file);
   
   let entryDepthString = if (Entry.get entryDepth)="" then "" else (" -d "^(Entry.get entryDepth)) in
   let entryPathString = if (Entry.get entryPath)="" then "" else (" -p "^(Entry.get entryPath)) in
   
   ignore(Sys.command(hlpsl2if^" --stdout "^esc^fileName^esc^" >"^out_name_if^" 2>"^error_if_file^" ; "^ofmc^" "^out_name_if^" "^sessionString^entryDepthString^entryPathString^" > "^out_name_ofmc^" 2>"^error_ofmc_file));
   
   Text.delete outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`End,[]);
   if(Sys.file_exists out_name_ofmc) then  (
   	let txt = readFile out_name_ofmc in
	if(txt<>"")
	then(
   		let (out_error, num_of_lines) = readFile_and_give_line_number error_if_file in
	 	let warning= (if(num_of_lines >2)  then ("\n----- Output error of if translator :\n\n"^out_error^"\n--------------- \n") else "") in
	Text.insert ~index:(`End,[]) ~text:(warning^txt) outText
	)
	else(
   		if(Sys.file_exists error_if_file) 
		then Text.insert ~index:(`End,[]) ~text:(("\n----- Output error of if translator :\n\n")^(readFile error_if_file)) outText;
		if(Sys.file_exists error_ofmc_file) 
		then Text.insert ~index:(`End,[]) ~text:(("\n\n\n----- Output error of OFMC :\n\n")^(readFile error_ofmc_file)) outText;
   		);
	);
   if(Sys.file_exists out_name_if) then ignore(Sys.remove out_name_if);
   if(Sys.file_exists out_name_ofmc) then ignore(Sys.remove out_name_ofmc);
   if(Sys.file_exists error_if_file) then ignore(Sys.remove error_if_file);
   if(Sys.file_exists error_ofmc_file) then ignore(Sys.remove error_ofmc_file);
   )
end;;


class atseOption optionFrame =
object
 val optionLabel : Widget.label Widget.widget = Label.create ~text:"Options" ~width:20 ~height:2 ~background:blue optionFrame
 val simplify : Widget.checkbutton Widget.widget = Checkbutton.create ~text:"Simplify            " ~width:20 ~height:2 ~borderwidth:1 optionFrame
 val mutable simplifyString : string = ""
 val untyped : Widget.checkbutton Widget.widget = Checkbutton.create ~text:"Untyped model" ~width:20 ~height:2 ~borderwidth:1 optionFrame
 val mutable untypedString : string = ""
 val verbose : Widget.checkbutton Widget.widget = Checkbutton.create ~text:"Verbose mode  " ~width:20 ~height:2 ~borderwidth:1 optionFrame
 val mutable verboseString : string = ""
 val search : Widget.label Widget.widget = Label.create ~text:"Search Algorithm" ~width:20 ~height:2 optionFrame
 val lst : Widget.listbox Widget.widget = Listbox.create ~height:2 ~background:selcol ~selectmode:`Single optionFrame
 (** Initialize the objects of this class *)
  method init : unit = (
     Checkbutton.configure simplify ~command:(fun () -> (if simplifyString="" then simplifyString <- " -light " else simplifyString <- ""));
     Checkbutton.configure untyped ~command:(fun () -> (if untypedString="" then untypedString <- " -notype " else untypedString <- ""));
     Checkbutton.configure verbose ~command:(fun () -> (if verboseString="" then verboseString <- " -vv " else verboseString <- ""));
     Listbox.insert lst ~index:`End ~texts:["Depth first"; "Breadth first"];
     Listbox.selection_set lst ~first:(`Num 0) ~last:(`Num 0);
     pack ~side:`Top [optionFrame];
     pack ~side:`Top [optionLabel];
     pack ~side:`Top [simplify];
     pack ~side:`Top [untyped];
     pack ~side:`Top [verbose];
     pack ~side:`Top [search];
     pack ~side:`Top [lst];)
 (** Destruct the objects of this class, and the graphical objects builds by the objects of this class *)
 method destroy : unit = ( destroy optionLabel; destroy simplify; 
		    destroy untyped; destroy verbose; 
		    destroy search; destroy lst; )
 (** Execute ATSE with the parameters of the user *)
 method execution (fileName : string) (outText : Widget.text Widget.widget) : unit = (
   let out_name_if = get_output_if fileName in
   let out_name_atse = (temp_dir^(Filename.basename fileName)^".atse") in
   let error_if_file = (out_name_if^".error") in
   let error_atse_file = (out_name_atse^".error") in
   if(Sys.file_exists out_name_if) then ignore(Sys.remove out_name_if);
   if(Sys.file_exists out_name_atse) then ignore(Sys.remove out_name_atse);
   if(Sys.file_exists error_if_file) then ignore(Sys.remove error_if_file);
   if(Sys.file_exists error_atse_file) then ignore(Sys.remove error_atse_file);
   
   let select = match (Listbox.curselection lst)
                with [(`Num num)] -> num
                 | _ -> -1 in
   let algo = match select
	      with 0 -> " -td "
               |    1 -> " -lr "
               |    _ -> "" in 
	       
   ignore(Sys.command(hlpsl2if^" --stdout "^esc^fileName^esc^" >"^out_name_if^" 2>"^error_if_file^" ; "^atse^" "^out_name_if^" "^simplifyString^untypedString^verboseString^algo^" > "^out_name_atse^" 2>"^error_atse_file));
   
   Text.delete outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`End,[]);
   if(Sys.file_exists out_name_atse) then  (
   	let txt = readFile out_name_atse in
   	if(Sys.file_exists error_if_file) 
	then (let (_,line_number)= (readFile_and_give_line_number error_if_file) in
		if(line_number > 2) 
		then (Text.insert ~index:(`End,[]) ~text:(("\n----- Output error of if translator :\n\n")^(readFile error_if_file)) outText;
			if(Sys.file_exists error_atse_file) 
			then Text.insert ~index:(`End,[]) ~text:(("\n\n\n----- Output error of ATSE :\n\n")^(readFile error_atse_file)) outText;
			if(Sys.file_exists out_name_atse) 
			then Text.insert ~index:(`End,[]) ~text:(("\n\n")^txt) outText;
			)
		else 	(if(Sys.file_exists out_name_atse) 
			then Text.insert ~index:(`End,[]) ~text:txt outText;)
	)
	else (if(Sys.file_exists out_name_atse) 
	    then Text.insert ~index:(`End,[]) ~text:txt outText;)
   );
   if(Sys.file_exists out_name_if) then ignore(Sys.remove out_name_if);
   if(Sys.file_exists out_name_atse) then ignore(Sys.remove out_name_atse);
   if(Sys.file_exists error_if_file) then ignore(Sys.remove error_if_file);
   if(Sys.file_exists error_atse_file) then ignore(Sys.remove error_atse_file);
   )
end;;


class satmcOption optionFrame =
object
 val optionLabel : Widget.label Widget.widget = Label.create ~text:"Options" ~width:20 ~height:2 ~background:blue optionFrame
 val frame2 : Widget.frame Widget.widget = Frame.create optionFrame
 val mutable solverLabel : Widget.label Widget.widget = Label.create ~text:"Solver : " ~height:2 optionFrame
 val mutable lst : Widget.listbox Widget.widget = Listbox.create ~height:2 ~background:selcol ~borderwidth:1 optionFrame
 val frame3 : Widget.frame Widget.widget = Frame.create optionFrame
 val mutable depthLabel : Widget.label Widget.widget = Label.create ~text:"Depth : " ~height:2 optionFrame
 val mutable depthEntry : Widget.entry Widget.widget = Entry.create ~width:20 ~borderwidth:1 optionFrame
 val abstraction : Widget.checkbutton Widget.widget = Checkbutton.create ~text:"Abstraction/Refinement   " ~width:20 ~height:2 ~borderwidth:1 optionFrame
 val mutable abstractionString : string = " --mutex=0  "
 val compound : Widget.checkbutton Widget.widget = Checkbutton.create ~text:"Compound Types                 " ~width:20 ~height:2 ~borderwidth:1 optionFrame
 val mutable compoundString : string = " --typed_model=strongly "
 val optimized : Widget.checkbutton Widget.widget = Checkbutton.create ~text:"Optimized intruder            " ~width:20 ~height:2 ~borderwidth:1 optionFrame
 val mutable optimizedString : string = " --oi=false "
 (** Initialize the objects of this class *)
  method init : unit = (
     destroy solverLabel;
     solverLabel <- Label.create ~text:"Solver : "  ~height:2 frame2;
     destroy lst;
     lst <- Listbox.create ~height:2 ~background:selcol frame2;
     destroy depthLabel;
     depthLabel <- Label.create ~text:"Depth : "  ~height:2 frame3;
     destroy depthEntry;
     depthEntry <- Entry.create ~width:20 ~background:selcol frame3;
     Listbox.insert lst ~index:`End ~texts:["Chaff"; "SIM" ; "SATO"];
     Listbox.selection_set lst ~first:(`Num 0) ~last:(`Num 0);
     Checkbutton.configure abstraction ~command:(fun () -> (if abstractionString=" --mutex=0  " then abstractionString <- " --mutex=1 " else abstractionString <- " --mutex=0  "));
     Checkbutton.configure compound ~command:(fun () -> (if compoundString=" --typed_model=strongly " then compoundString <- " --typed_model=yes " else compoundString <- " --typed_model=strongly "));
     Checkbutton.configure optimized ~command:(fun () -> (if optimizedString=" --oi=false " then optimizedString <- " --oi=true " else optimizedString <- " --oi=false "));
     pack ~side:`Top [optionFrame];
     pack ~side:`Top [optionLabel];
     pack ~side:`Top [frame2];
     pack ~side:`Left [solverLabel];
     pack ~side:`Right [lst];
     pack ~side:`Top [frame3];
     pack ~side:`Left [depthLabel];
     pack ~side:`Right [depthEntry];
     pack ~side:`Top [abstraction];
     pack ~side:`Top [compound];
     pack ~side:`Top [optimized];)
(** Destruct the objects of this class, and the graphical objects builds by the objects of this class *)
 method destroy : unit = ( destroy optionLabel; destroy frame2; 
		    destroy solverLabel; destroy lst; 
		    destroy frame3; destroy depthLabel;
		    destroy depthEntry; destroy abstraction;
		    destroy compound; destroy optimized;)
 (** Execute SATMC with the parameters of the user *)
 method execution (fileName : string) (outText : Widget.text Widget.widget) : unit = (
   let out_name_if = get_output_if fileName in
   let out_name_satmc = (temp_dir^(Filename.basename fileName)^".satmc") in
   let error_if_file = (out_name_if^".error") in
   let error_satmc_file = (out_name_satmc^".error") in
   if(Sys.file_exists out_name_if) then ignore(Sys.remove out_name_if);
   if(Sys.file_exists out_name_satmc) then ignore(Sys.remove out_name_satmc);
   if(Sys.file_exists error_if_file) then ignore(Sys.remove error_if_file);
   if(Sys.file_exists error_satmc_file) then ignore(Sys.remove error_satmc_file);
   
   let select = match (Listbox.curselection lst)
                with [(`Num num)] -> num
                 | _ -> -1 in
   let algo = match select
	      with 0 -> " --solver=chaff "
               |    1 -> " --solver=sim "
               |    2 -> " --solver=sato "
               |    _ -> "" in 
   let depthString = if (Entry.get depthEntry)="" then "" else (" --max="^(Entry.get depthEntry)) in
   
   ignore(Sys.command(hlpsl2if^" --stdout "^esc^fileName^esc^" >"^out_name_if^" 2>"^error_if_file^" ; "^satmc^" "^out_name_if^" "^compoundString^" --output="^(Filename.dirname fileName)^abstractionString^optimizedString^depthString^algo^" > "^out_name_satmc^" 2>"^error_satmc_file));
   
   Text.delete outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`End,[]);
   if(Sys.file_exists out_name_satmc) then  (
   	let txt = readFile out_name_satmc in
   	if(Sys.file_exists error_if_file) 
	then (let (_,line_number)= (readFile_and_give_line_number error_if_file) in
		if(line_number > 2) 
		then (Text.insert ~index:(`End,[]) ~text:(("\n----- Output error of if translator :\n\n")^(readFile error_if_file)) outText;
			if(Sys.file_exists error_satmc_file) 
			then Text.insert ~index:(`End,[]) ~text:(("\n\n\n----- Output error of SATMC :\n\n")^(readFile error_satmc_file)) outText;
			if(Sys.file_exists out_name_satmc) 
			then Text.insert ~index:(`End,[]) ~text:(("\n\n")^txt) outText;
			)
		else 	(if(Sys.file_exists out_name_satmc) 
			then Text.insert ~index:(`End,[]) ~text:txt outText;)
	)
	else (if(Sys.file_exists out_name_satmc) 
	    then Text.insert ~index:(`End,[]) ~text:txt outText;)
   );
   if(Sys.file_exists out_name_if) then ignore(Sys.remove out_name_if);
   if(Sys.file_exists out_name_satmc) then ignore(Sys.remove out_name_satmc);
   if(Sys.file_exists error_if_file) then ignore(Sys.remove error_if_file);
   if(Sys.file_exists error_satmc_file) then ignore(Sys.remove error_satmc_file);
   )
end;;


class ta4spOption optionFrame =
object
 val optionLabel : Widget.label Widget.widget = Label.create ~text:"Options" ~width:20 ~height:2 ~background:blue optionFrame
 val verificationLabel : Widget.label Widget.widget = Label.create ~text:"Verification Model" ~width:20 ~height:2 optionFrame
 val agent : Widget.checkbutton Widget.widget = Checkbutton.create ~text:"Two Agent Only" ~width:20 ~height:2 ~borderwidth:1 optionFrame
 val mutable agentString : string = ""
 val knowledgeLabel : Widget.label Widget.widget = Label.create ~text:"Intruder Knowledge" ~width:20 ~height:2 optionFrame
 val tv  : Textvariable.textVariable= Textvariable.create ()
 val mutable rads : Widget.radiobutton Widget.widget list = [] 
 val frame2 : Widget.frame Widget.widget = Frame.create optionFrame
 val mutable levelLabel : Widget.label Widget.widget = Label.create ~text:"Level : " ~width:20 ~height:2 optionFrame
 val mutable levelEntry : Widget.entry Widget.widget = Entry.create ~width:20 ~borderwidth:1 ~background:defcol ~state:`Disabled optionFrame
 (** Initialize the objects of this class *)
 method init : unit = (
        Textvariable.set tv "1";
     rads <- List.map
        (fun (t, v) -> Radiobutton.create ~text:t ~value:v ~variable:tv ~borderwidth:1 optionFrame)
        [("Over-Approximation  ", "1"); ("Under-Approximation","2")];
     match rads with []          -> ();
                     |[x]       -> ();
                     |(x::y::_) -> (Radiobutton.select x; 
                                   Radiobutton.configure x ~command:(fun () -> Entry.configure levelEntry  ~background:defcol ~state:`Disabled);
                                   Radiobutton.configure y ~command:(fun () -> Entry.configure levelEntry  ~background:selcol ~state:`Normal););

                                 
     destroy levelLabel;
     levelLabel <- Label.create ~text:"Level : " ~width:20 ~height:2 frame2;
     destroy levelEntry;
     levelEntry <- Entry.create ~width:20 ~background:defcol ~state:`Disabled frame2;
     Checkbutton.configure agent ~command:(fun () -> (if agentString="" then agentString <- " --2AgentsOnly " else agentString <- ""));
     pack ~side:`Top [optionFrame];
     pack ~side:`Top [optionLabel];
     pack ~side:`Top [verificationLabel];
     pack ~side:`Top [agent];
     pack ~side:`Top [knowledgeLabel];
     pack rads;
     pack ~side:`Top [frame2];
     pack ~side:`Left [levelLabel];
     pack ~side:`Right [levelEntry];)
 (** Destruct the objects of this class, and the graphical objects builds by the objects of this class *)
 method destroy : unit = ( destroy optionLabel; destroy verificationLabel; 
                    List.iter (fun t -> destroy t) rads;
		    destroy agent; destroy knowledgeLabel; 
		    destroy frame2; destroy levelLabel;
		    destroy levelEntry;)
 (** Execute TA4SP with the parameters of the user *)
 method execution (fileName : string) (outText : Widget.text Widget.widget) : unit = (
   let out_name_if = get_output_if fileName in
   let out_name_ta4sp = (temp_dir^(Filename.basename fileName)^".ta4sp") in
   let error_if_file = (out_name_if^".error") in
   let error_ta4sp_file = (out_name_ta4sp^".error") in
   if(Sys.file_exists out_name_if) then ignore(Sys.remove out_name_if);
   if(Sys.file_exists out_name_ta4sp) then ignore(Sys.remove out_name_ta4sp);
   if(Sys.file_exists error_if_file) then ignore(Sys.remove error_if_file);
   if(Sys.file_exists error_ta4sp_file) then ignore(Sys.remove error_ta4sp_file);
   
   let select = match (Textvariable.get tv)
                with "1" -> ""
                 | "2" -> (if (Entry.get levelEntry)="" then " --level=1 " else (" --level="^(Entry.get levelEntry)))
                 | _ -> "" in
		 
   ignore(Sys.command(hlpsl2if^" --stdout "^esc^fileName^esc^" >"^out_name_if^" 2>"^error_if_file^" ; "^ta4sp^" "^out_name_if^" "^agentString^select^" > "^out_name_ta4sp^" 2>"^error_ta4sp_file));
   
   Text.delete outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`End,[]);
   if(Sys.file_exists out_name_ta4sp) then  (
   	let txt = readFile out_name_ta4sp in
	if(txt<>"")
	then(
   		let (out_error, num_of_lines) = readFile_and_give_line_number error_if_file in
	 	let warning= (if(num_of_lines >2)  then ("\n----- Output error of if translator :\n\n"^out_error^"\n--------------- \n") else "") in
	Text.insert ~index:(`End,[]) ~text:(warning^txt) outText
	)
	else(
   		if(Sys.file_exists error_if_file) 
		then Text.insert ~index:(`End,[]) ~text:(("\n----- Output error of if translator :\n\n")^(readFile error_if_file)) outText;
		if(Sys.file_exists error_ta4sp_file) 
		then Text.insert ~index:(`End,[]) ~text:(("\n\n\n----- Output error of TA4SP :\n\n")^(readFile error_ta4sp_file)) outText;
   		);
	);
   if(Sys.file_exists out_name_if) then ignore(Sys.remove out_name_if);
   if(Sys.file_exists out_name_ta4sp) then ignore(Sys.remove out_name_ta4sp);
   if(Sys.file_exists error_if_file) then ignore(Sys.remove error_if_file);
   if(Sys.file_exists error_ta4sp_file) then ignore(Sys.remove error_ta4sp_file);
   )
end;;

(*fichier ouvert*)
let fileName = (ref "" : string Pervasives.ref);;

(* Parsing the arguments of the compiler, for detecting options and
   possible mistakes.
*)

let is_input = ref false in
(Arg.parse []
   (fun s ->
     if !is_input then
       raise (Arg.Bad "error: multiple input files")
     else
       (is_input := true;
	fileName := s))
   "Usage: verif [file.hlpsl]"
);;

let _ =
configure();
(* Title setting *)
Wm.title_set root "SPAN - Protocol Verification" ;
Wm.resizable_set root ~width:true ~height:true ;

(* Base frame *)
let base = Frame.create root in
(*Wm.resizable_set base ~width:true ~height:true ;*)
pack ~expand:true ~fill:`Both [base];

let lock_save = ref true in

(* Menu bar *)
let bar = Frame.create ~borderwidth:1 ~relief:`Raised base in 
pack ~fill:`X [bar];

let commandFrame = Frame.create ~relief:`Groove ~borderwidth:2  base in
let commandToolFrame = Frame.create commandFrame in

let optionFrame = Frame.create commandToolFrame  in
let gOptionFrame = ref (new hlpsl2ifOption optionFrame) in

let middleFrame = Canvas.create base in
let outText = Text.create ~width:150 ~height:35 ~relief:`Ridge ~borderwidth:1 ~selectborderwidth:3 ~background:selcol middleFrame in


let sb = Scrollbar.create ~background:blue ~borderwidth:1 ~orient:`Vertical middleFrame in
let scroll_link sb outText =
    Text.configure ~yscrollcommand:(Scrollbar.set sb) outText;
    Scrollbar.configure ~command:(Text.yview outText) sb in
  scroll_link sb outText; 
let fileFrame =  Frame.create ~relief:`Ridge ~borderwidth:1  commandFrame  in
let saveButton = Button.create  ~width:10 ~height:2 ~borderwidth:1 fileFrame ~text:"Save file" ~command:(fun () -> 
   (let tmp = (Text.get outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`End,[])) in
   let txt = if(tmp<>"" && (String.sub tmp 0 1)<>"\n") then tmp else (String.sub tmp 1 ((String.length tmp)-1)) in
   if ((!fileName)="")
   then ()
   else saveFile (!fileName) txt )) in
let fileButton = Button.create  ~width:10 ~height:2 ~borderwidth:1 fileFrame ~text:"View file" ~command:(fun () -> 
   Text.delete outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`End,[]) ;
   Text.delete outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`Linechar(0, 0),[]) ;
   if ((!fileName)="")
   then ()
   else (lock_save := true ;
	Button.configure saveButton ~state:`Active;
   	Text.insert ~index:(`End,[]) ~text:(readFile (!fileName)) outText)) in
let simulationButton = Button.create  ~width:10 ~height:2 ~borderwidth:1 fileFrame ~text:"Protocol\nsimulation" ~command:(fun () -> 
   if ((!fileName)="")
   then ()
   else (
   let error_sim_file = (temp_dir^(Filename.basename (!fileName))^".sim"^".error") in
   if(Sys.file_exists error_sim_file) then ignore(Sys.remove error_sim_file);
   
   ignore(Sys.command(hlpsl2scenario^" "^esc^(!fileName)^esc^" 2> "^error_sim_file^" &"));

  Unix.sleep 1;
   if(Sys.file_exists error_sim_file) then(
      	let txt = readFile error_sim_file in
	if(txt<>"") then ( 
		lock_save := false ; 
		Button.configure saveButton ~state:`Disabled;
		Text.delete outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`End,[]);
		Text.insert ~index:(`End,[]) ~text:(("\n----- Output error of protocol simulation :\n\n")^txt) outText;
		);
   	ignore(Sys.remove error_sim_file);
   	)
   )) in
let intruderButton = Button.create  ~width:10 ~height:2 ~borderwidth:1 fileFrame ~text:"Intruder\nsimulation" ~command:(fun () -> 
   if ((!fileName)="")
   then ()
   else (
   let error_sim_file = (temp_dir^(Filename.basename (!fileName))^".sim"^".error") in
   if(Sys.file_exists error_sim_file) then ignore(Sys.remove error_sim_file);
   
   ignore(Sys.command(hlpsl2scenario^" --intruder "^esc^(!fileName)^esc^(*" 2> "^error_sim_file^*)" &"));

   Unix.sleep 1;
   if(Sys.file_exists error_sim_file) then(
      	let txt = readFile error_sim_file in
	if(txt<>"") then ( 
		lock_save := false ; 
		Button.configure saveButton ~state:`Disabled;
		Text.delete outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`End,[]);
		Text.insert ~index:(`End,[]) ~text:(("\n----- Output error of protocol simulation :\n\n")^txt) outText;
		);
   	ignore(Sys.remove error_sim_file);
   	)
   )) in
(* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% *)
let tool = Frame.create commandToolFrame in
let tool1 = Frame.create tool in
let labelHlpsl2if = Label.create ~text:"HLPSL" ~width:8 ~height:2 tool1 in
let tool2 = Frame.create tool in

let toolHlpsl2if = Checkbutton.create ~text:"HLPSL2IF" ~width:8 ~height:2 ~borderwidth:1 ~background:blue  ~indicatoron:false  ~selectcolor:clicol  tool2 in
let tool3 = Frame.create tool in
let labelIf = Label.create ~text:"IF" ~width:8 ~height:2 tool3 in
let tool4 = Frame.create tool in
let toolOFMC = Checkbutton.create ~text:"OFMC" ~width:8 ~height:2 ~borderwidth:1 ~background:blue  ~indicatoron:false ~selectcolor:clicol  tool4 in
let toolATSE = Checkbutton.create ~text:"ATSE" ~width:8 ~height:2 ~borderwidth:1 ~background:blue  ~indicatoron:false ~selectcolor:clicol  tool4 in
let toolSATMC = Checkbutton.create ~text:"SATMC" ~width:8 ~height:2 ~borderwidth:1 ~background:blue  ~indicatoron:false ~selectcolor:clicol  tool4 in
let toolTA4SP = Checkbutton.create ~text:"TA4SP" ~width:8 ~height:2 ~borderwidth:1 ~background:blue  ~indicatoron:false ~selectcolor:clicol  tool4 in

let unchoose = List.iter (fun t-> Checkbutton.configure t ~state:`Normal ~takefocus:false; Checkbutton.deselect t) in
Checkbutton.configure toolHlpsl2if ~command:(fun () -> ignore(unchoose [toolOFMC ; toolATSE ; toolSATMC ; toolTA4SP]; (!gOptionFrame)#destroy; gOptionFrame := new hlpsl2ifOption optionFrame ; (!gOptionFrame)#init ););
Checkbutton.configure toolOFMC ~command:(fun () -> ignore(unchoose [toolHlpsl2if ; toolATSE ; toolSATMC ; toolTA4SP]; (!gOptionFrame)#destroy; gOptionFrame := new ofmcOption optionFrame; (!gOptionFrame)#init; ););
Checkbutton.configure toolATSE ~command:(fun () -> ignore(unchoose [toolHlpsl2if ; toolOFMC ; toolSATMC ; toolTA4SP]; (!gOptionFrame)#destroy; gOptionFrame := new atseOption optionFrame; (!gOptionFrame)#init; ););
Checkbutton.configure toolSATMC ~command:(fun () -> ignore(unchoose [toolHlpsl2if ; toolOFMC ; toolATSE ; toolTA4SP]; (!gOptionFrame)#destroy; gOptionFrame := new satmcOption optionFrame; (!gOptionFrame)#init; );) ;
Checkbutton.configure toolTA4SP ~command:(fun () -> ignore(unchoose [toolHlpsl2if ; toolOFMC ; toolATSE ; toolSATMC]; (!gOptionFrame)#destroy; gOptionFrame := new ta4spOption optionFrame; (!gOptionFrame)#init; ););
let blankFrame1 = Frame.create  tool in
let toolLabel = Label.create ~text:"Tools" ~background:blue ~width:20 ~height:2 blankFrame1 in
let blankFrame2 = Frame.create  ~width:120 ~height:50 tool in


let executeFrame = Frame.create ~relief:`Solid ~borderwidth:1  commandToolFrame in
let executeLabel1 = Label.create ~text:"Choose Tool option and" executeFrame in
let executeLabel2 = Label.create ~text:"press execute" executeFrame in
let executeButton = Button.create ~background:clicol ~text:"Execute" ~width:16 ~height:1 ~borderwidth:1 
										~command:(fun () -> (
											     match (!fileName) 
											     with "" -> ()
											     | _ -> (
												lock_save := false ;
												Button.configure saveButton ~state:`Disabled;
												(!gOptionFrame)#execution (!fileName) outText))) executeFrame in
let types = [
     {typename="Hlpsl Files";       extensions=[".hlpsl"]; mactypes=["TEXT"]};
     {typename="Text Files";       extensions=[".txt"; ".text"]; mactypes=["TEXT"]};
     {typename="All Files";        extensions=["*"];       mactypes=[""]}
 ] in
(* Menu and Menubutton *)
let meb = Menubutton.create ~text:"File" ~font:"guifont" ~activebackground:blue bar in
let men = Menu.create meb ~activebackground:blue ~borderwidth:1 ~relief:`Groove in
Menu.add_command ~label:"Open File..." ~command:(fun () -> (   let file= (!fileName) in 
							       fileName:=Tk.getOpenFile() ~defaultextension:".hlpsl" ~filetypes:types ~initialdir:(Filename.dirname (!fileName)) ~title:"Open a protocol file";
							       if (!fileName)="" then fileName:=file else ();
							       Wm.title_set root ("SPAN - Protocol Verification : "^(Filename.basename (!fileName)));
							       Text.delete outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`End,[]) ;
							       if ((!fileName)="")
							       then ()
							       else 
							       (lock_save := true ;
								Button.configure saveButton ~state:`Active;
							        Text.insert ~index:(`End,[]) ~text:(readFile (!fileName)) outText;))) men;
Menu.add_command ~label:"Save" ~command:(fun () -> 
					(let tmp = (Text.get outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`End,[])) in
   					let txt = if(tmp<>"" && (String.sub tmp 0 1)<>"\n") then tmp else (String.sub tmp 1 ((String.length tmp)-1)) in
					if ((!fileName)="" && (!lock_save))
					then ()
					else saveFile (!fileName) txt )) men;
Menu.add_command ~label:"Save as..." ~command:(fun () -> 
   (let file= (!fileName) in
    fileName:=Tk.getSaveFile() ~defaultextension:".hlpsl" ~filetypes:types ~initialdir:(Filename.dirname (!fileName)) ~title:"Save a protocol file";
    if (!fileName)="" then fileName:=file else ();
    let offset = 1+(String.rindex (!fileName) '/') in
    Wm.title_set root ("SPAN - Protocol Verification : "^(String.sub (!fileName) offset ((String.length (!fileName))-offset) ));
   let tmp = (Text.get outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`End,[])) in
   let txt = if(tmp<>"" && (String.sub tmp 0 1)<>"\n") then tmp else (String.sub tmp 1 ((String.length tmp)-1)) in
   if ((!fileName)="")
   then ()
   else saveFile (!fileName) txt )) men;
Menu.add_separator men;
Menu.add_command ~label:"Quit" ~command:(fun () -> closeTk (); exit 0) men;
Menubutton.configure ~menu:men meb;

if ((!fileName)<>"")
then (   Wm.title_set root ("SPAN - Protocol Verification : "^(Filename.basename (!fileName)));
	 Text.delete outText ~start:(`Linechar(0, 0),[`Linestart]) ~stop:(`End,[]) ;
	 Text.insert ~index:(`End,[]) ~text:(readFile (!fileName)) outText;)
else();


    pack ~side:`Left [meb];

    pack ~side:`Left ~fill:`Both  [tool];
    pack ~side:`Top [blankFrame1];
    pack ~side:`Top [toolLabel];
    pack ~side:`Top [tool1];
    pack ~side:`Bottom [labelHlpsl2if];
    pack ~side:`Top [tool2];
    pack ~side:`Left [toolHlpsl2if];
    pack ~side:`Top [tool3];
    pack ~side:`Left [labelIf];
    pack ~side:`Top [tool4];
    pack ~side:`Left [toolOFMC];
    pack ~side:`Left [toolATSE];
    pack ~side:`Left [toolSATMC];
    pack ~side:`Left [toolTA4SP];
    
    pack ~side:`Left ~fill:`X [executeFrame];
    pack ~side:`Top [executeLabel1];
    pack ~side:`Top [executeLabel2];
    pack ~side:`Top [executeButton];
    
    pack ~side:`Left  [optionFrame];
    pack ~side:`Bottom [blankFrame2];
    
        
    pack ~side:`Bottom ~fill:`Both   [commandToolFrame];
    
    pack ~side:`Left [saveButton];
    pack ~side:`Left [fileButton];
    pack ~side:`Left [simulationButton];
    pack ~side:`Left [intruderButton];
    pack ~side:`Bottom [fileFrame];
    
    pack ~side:`Bottom ~fill:`Both    [commandFrame];
    

    pack ~side:`Top ~fill:`Both ~expand:true [middleFrame];
    pack ~side:`Right ~fill:`Y [sb];
    pack ~side:`Left ~fill:`Both ~expand:true [outText];

Printexc.print mainLoop();
